from __future__ import annotations


USAGE = """
A tool to execute python code.
You can (and should) use the provided stubs as async function calls
if you need to.
Write an async main function which returns the result if neccessary.
DONT write placeholders. DONT run the function yourself. just write the function.

Example:
async def main():
    result_1 = await provided_function()
    result_2 = await provided_function_2("some_arg")
    return result_1 + result_2

You may also use loops, conditionals etc. if neccessary.

"""
